package mngr

import (
	"github.com/hyperledger/fabric/common/ledger/testutil"
	"github.com/stretchr/testify/mock"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
	"reflect"
	"testing"
)

func TestSystemPreferenceManagerImpl_AddPreference(t *testing.T) {

	chaincode.InitDB(t)
	dbAdapter, manager, _ := chaincode.GetManagerDB(t)
	systemPreferenceManager := GetSystemPreferenceManager(manager)

	thisTest := tests{
		{
			name:    "ALL READY EXSIST",
			manager: systemPreferenceManager,
			model: &preferenceCC.Preference{
				Key:    "SystemID",
				Fields: chaincode.Fields{"SystemID": "44"}},
			willAdd: &preferenceCC.Preference{
				Key:    "SystemID",
				Fields: chaincode.Fields{"SystemID": "44"}},
			wantErr:        true,
			errDescription: "ALREADY_EXISTS",
		},
		{
			name:    "Key is empty",
			manager: systemPreferenceManager,
			model: &preferenceCC.Preference{
				Key:    "",
				Fields: chaincode.Fields{"SystemID": "44"}},
			wantErr:        true,
			errDescription: "key is empty",
		},
		{
			name:    "all good",
			manager: systemPreferenceManager,
			model: &preferenceCC.Preference{
				Key:    "SystemPreference",
				Fields: chaincode.Fields{"SystemID": "44"}},
			want: &preferenceCC.Preference{
				Key:    "SystemPreference",
				Fields: chaincode.Fields{"SystemID": "44"}},
			wantErr:        false,
			errDescription: "",
		},
	}

	for _, tt := range thisTest {

		chaincode.CleanupDBTest(t)

		dbAdapter.On("PutState", mock.Anything).Return().Once()
		dbAdapter.On("GetState", mock.Anything).Return().Once()

		t.Run(tt.name, func(t *testing.T) {

			if tt.willAdd != nil {

				err := mockInvokeUserCC(tt.willAdd.Key, manager, dbAdapter)
				if err != nil {
					t.Errorf("mockInvokeUserCC: %s", err.Error())
					return
				}
				_, err = tt.manager.AddPreference(tt.willAdd)
				if err != nil {
					chaincode.GlobalLogger.Errorf(err.Error())
				}
			}

			err := mockInvokeUserCC(tt.model.Key, manager, dbAdapter)
			if err != nil {
				t.Errorf("mockInvokeUserCC: %s", err.Error())
				return
			}
			testutil.AssertNoError(t, err, "")
			got, err := tt.manager.AddPreference(tt.model)

			if (err != nil) && tt.wantErr == true {
				if err.Error() == tt.errDescription {
					chaincode.GlobalLogger.Infof("Ошибка доказана! " + err.Error())
				} else {
					t.Errorf("SystemPreferenceManagerImpl.AddPreference() error = %v, wantErr %v", err, tt.wantErr)
				}
			} else if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("SystemPreferenceManagerImpl.AddPreference() = %v, want %v", got, tt.want)
			}
		})
	}
}
