package mngr

import (
	"github.com/hyperledger/fabric/common/ledger/testutil"
	"github.com/stretchr/testify/mock"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"reflect"
	"testing"
)

func TestSystemPreferenceManagerImpl_GetPreference(t *testing.T) {

	chaincode.InitDB(t)
	dbAdapter, manager, _ := chaincode.GetManagerDB(t)
	systemPreferenceManager := GetSystemPreferenceManager(manager)

	test := tests{

		{
			name:           "requests ID is empty",
			manager:        systemPreferenceManager,
			requestsId:     &requests.Id{Id: ""},
			wantErr:        true,
			errDescription: "key is empty",
		},
		{
			name:       "Key model is empty",
			manager:    systemPreferenceManager,
			requestsId: &requests.Id{Id: "UserID"},
			willAdd: &preferenceCC.Preference{
				Key:    "",
				Fields: chaincode.Fields{"SystemID": "44"}},
			wantErr:        true,
			errDescription: "key is empty",
		},
		{
			name:           "not found",
			manager:        systemPreferenceManager,
			requestsId:     &requests.Id{Id: "UserID"},
			want:           nil,
			wantErr:        true,
			errDescription: "NOT_FOUND",
		},
		{
			name:           "all good",
			manager:        systemPreferenceManager,
			requestsId:     &requests.Id{Id: "SystemID"},
			wantErr:        false,
			errDescription: "",
			willAdd: &preferenceCC.Preference{
				Key:    "SystemID",
				Fields: chaincode.Fields{"SystemID": "44"}},
			want: &preferenceCC.Preference{
				Key:    "SystemID",
				Fields: chaincode.Fields{"SystemID": "44"}},
		},
	}

	for _, tt := range test {
		chaincode.CleanupDBTest(t)

		t.Run(tt.name, func(t *testing.T) {

			dbAdapter.On("PutState", mock.Anything).Return().Once()
			dbAdapter.On("GetState", mock.Anything).Return().Once()

			if tt.willAdd != nil {
				err := mockInvokeUserCC(tt.willAdd.Key, manager, dbAdapter)
				if err != nil {
					t.Errorf("mockInvokeUserCC: %s", err.Error())
					return
				}
				testutil.AssertNoError(t, err, "")
				got, err := tt.manager.AddPreference(tt.willAdd)
				if (err != nil) && tt.wantErr == true {
					if err.Error() == tt.errDescription {
						chaincode.GlobalLogger.Infof("Ошибка доказана! " + err.Error())
						return
					} else {
						t.Errorf("SystemPreferenceManagerImpl.GetPreference() error = %v, wantErr %v", err, tt.wantErr)
						return
					}
				} else if !reflect.DeepEqual(got, tt.want) {
					t.Errorf("SystemPreferenceManagerImpl.UpdatePreference() = %v, want %v", got, tt.want)
					return
				}
			}

			got, err := tt.manager.GetPreference(tt.requestsId)

			if (err != nil) && tt.wantErr == true {
				if err.Error() == tt.errDescription {
					chaincode.GlobalLogger.Infof("Ошибка доказана! " + err.Error())
				} else {
					t.Errorf("SystemPreferenceManagerImpl.GetPreference() error = %v, wantErr %v", err, tt.wantErr)
				}
			} else if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("SystemPreferenceManagerImpl.GetPreference() = %v, want %v", got, tt.want)
			}
		})
	}
}
