package mngr

import (
	"github.com/hyperledger/fabric/common/ledger/testutil"
	"github.com/stretchr/testify/mock"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/model/preferenceCC"
	"reflect"
	"testing"
)

func TestSystemPreferenceManagerImpl_UpdatePreference(t *testing.T) {

	chaincode.InitDB(t)
	dbAdapter, manager, _ := chaincode.GetManagerDB(t)
	userPreferenceManager := GetSystemPreferenceManager(manager)

	tests := tests{
		{
			name:    "Key is empty",
			manager: userPreferenceManager,
			model: &preferenceCC.Preference{
				Key:    "",
				Fields: chaincode.Fields{"SystemID": "44"}},
			wantErr:        true,
			errDescription: "key is empty",
		},
		{
			name:           "not found",
			manager:        userPreferenceManager,
			want:           nil,
			wantErr:        true,
			errDescription: "NOT_FOUND",
			model: &preferenceCC.Preference{
				Key:    "SystemID",
				Fields: chaincode.Fields{"SystemID": "44"}},
		},
		{
			name:           "all good",
			manager:        userPreferenceManager,
			wantErr:        false,
			errDescription: "NOT_FOUND",
			model: &preferenceCC.Preference{
				Key:    "SystemPreference",
				Fields: chaincode.Fields{"SystemID": "44"}},
			want: &preferenceCC.Preference{
				Key:    "SystemPreference",
				Fields: chaincode.Fields{"SystemID": "44"}},
			willAdd: &preferenceCC.Preference{
				Key:    "SystemPreference",
				Fields: chaincode.Fields{"SystemID": "44"}},
		},
	}

	for _, tt := range tests {

		chaincode.CleanupDBTest(t)

		t.Run(tt.name, func(t *testing.T) {

			dbAdapter.On("PutState", mock.Anything).Return().Once()
			dbAdapter.On("GetState", mock.Anything).Return().Once()

			if tt.willAdd != nil {
				err := mockInvokeUserCC(tt.willAdd.Key, manager, dbAdapter)
				if err != nil {
					t.Errorf("mockInvokeUserCC: %s", err.Error())
					return
				}
				testutil.AssertNoError(t, err, "")
				got, err := tt.manager.AddPreference(tt.willAdd)
				if (err != nil) && tt.wantErr == true {
					if err.Error() == tt.errDescription {
						chaincode.GlobalLogger.Infof("Ошибка доказана! " + err.Error())
						return
					} else {
						t.Errorf("SystemPreferenceManagerImpl.GetPreference() error = %v, wantErr %v", err, tt.wantErr)
						return
					}
				} else if !reflect.DeepEqual(got, tt.want) {
					t.Errorf("SystemPreferenceManagerImpl.UpdatePreference() = %v, want %v", got, tt.want)
					return
				}
			}

			got, err := tt.manager.UpdatePreference(tt.model)
			if (err != nil) && tt.wantErr == true {
				if err.Error() == tt.errDescription {
					chaincode.GlobalLogger.Infof("Ошибка доказана! " + err.Error())
				} else {
					t.Errorf("SystemPreferenceManagerImpl.UpdatePreference() error = %v, wantErr %v", err, tt.wantErr)
				}
			} else if !reflect.DeepEqual(got, tt.want) {
				t.Errorf("SystemPreferenceManagerImpl.UpdatePreference() = %v, want %v", got, tt.want)
			}
		})
	}
}
