package chaincode

import (
	"encoding/json"
	"github.com/sirupsen/logrus"
	"fmt"
	"errors"
)

var logger = logrus.WithFields(logrus.Fields{
	"module":  "common",
	"package": "chaincode",
})

func GetSelector(data interface{}) (query string, err error) {
	if data == nil {
		query = "{\"selector\": {\"_id\": {\"$gt\": null}}}"
	} else {
		logger.Info("getSelector")
		queryByte, err := json.Marshal(struct {
			Selector interface{} `json:"selector,omitempty"`
		}{
			Selector: data,
		})

		if err != nil {
			return "", err
		}
		query = string(queryByte)
	}

	logger.Infof("query: %s", query)
	return
}

func FindOneByQuery(queryObject interface{}, adapter StubInterfaceAdapter) ([]byte, error) {
	logger.Info("findOneByQuery")
	query, err := GetSelector(queryObject)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to create query. Error: %s", err)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	logger.Info("query: ", query)

	it, err := adapter.GetQueryResult(query)
	if err != nil {
		errMsg := fmt.Sprintf("Failed to query %s. Error: %s", query, err)
		logger.Error(errMsg)
		return nil, errors.New(errMsg)
	}

	defer it.Close() // Do not forget to close iterator once get used
	for it.HasNext() {
		record, err := it.Next()
		if err != nil {
			errMsg := fmt.Sprintf("Failed to read record from query iterator for query %s. Error: %s", query, err)
			logger.Error(errMsg)
			return nil, errors.New(errMsg)
		}

		return record.Value, nil
	}

	return nil, errors.New("not found")
}
