package chaincode

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	pb "github.com/hyperledger/fabric/protos/peer"
)

type IManager interface {
	GetTime() int64
	SetAdapter(new StubInterfaceAdapter)
	SetTimeGetter(new TimeGetter)
	GetAdapter() StubInterfaceAdapter
	GetTimeGetter() TimeGetter
	GetState(key string) ([]byte, error)
	PutState(key string, value []byte) error
	DelState(key string) error
	// TODO REMOVE >>> DEPRECATED <<<
	GetQueryResult(query string) (shim.StateQueryIteratorInterface, error)
	InvokeChaincode(chaincodeName string, args [][]byte, channel string) pb.Response
	GetTxID() string
	Save(obj ValidObject) (ValidObject, error)
	IsExists(obj ValidObject) (bool, error)
	GetIfExists(newObject ValidObject) (bool, error)
	AddObject(newObject ValidObject) (ValidObject, error)
	UpdateObject(newObject ValidObject) (ValidObject, error)
	GetObject(expectedObject ValidObject, keys ...string) (ValidObject, error)
	GetObjectByKeys(expectedObject ValidObject, keys []string) (ValidObject, error)
	SignedInvokeChaincode(channelName string, chaincodeName string, functionName string, arg interface{}) pb.Response
	PrepareInvokeChaincodeParam(channelName string, chaincodeName string, functionName string, arg interface{}) ([][]byte, error)
	InvokeChaincode_GetUser(UserID string) pb.Response
	InvokeChaincode_VerifyUserSignature(arg interface{}) pb.Response
}
