package chaincode

import (
	"github.com/hyperledger/fabric/common/ledger/testutil"
	"github.com/hyperledger/fabric/core/ledger/kvledger/txmgmt/statedb/statecouchdb"
	ledgertestutil "github.com/hyperledger/fabric/core/ledger/testutil"
	"github.com/spf13/viper"
	"testing"
	"time"
)

func InitDB(t *testing.T) {
	// Read the core.yaml file for default config.
	ledgertestutil.SetupCoreYAMLConfig()
	viper.Set("peer.fileSystemPath", "/tmp/fabric/ledgertests/kvledger/txmgmt/statedb/statecouchdb")
	// Switch to CouchDB
	viper.Set("ledger.state.stateDatabase", "CouchDB")
	// both vagrant and CI have couchdb configured at host "couchdb"
	viper.Set("ledger.state.couchDBConfig.couchDBAddress", "couchdb:5984")
	// Replace with correct username/password such as
	// admin/admin if user security is enabled on couchdb.
	viper.Set("ledger.state.couchDBConfig.username", "")
	viper.Set("ledger.state.couchDBConfig.password", "")
	viper.Set("ledger.state.couchDBConfig.maxRetries", 3)
	viper.Set("ledger.state.couchDBConfig.maxRetriesOnStartup", 10)
	viper.Set("ledger.state.couchDBConfig.requestTimeout", time.Second*35)
	//run the actual test
	//revert to default goleveldb
	viper.Set("ledger.state.stateDatabase", "goleveldb")
}

func GetManagerDB(t *testing.T) (StubInterfaceAdapterDBMock, ManagerImpl) {
	//конектим к коуч дб
	env := statecouchdb.NewTestVDBEnv(t)
	//очищает только с приставкой "_"
	env.Cleanup("statecouchdb_")
	db, err := env.DBProvider.GetDBHandle("statecouchdb")
	testutil.AssertNoError(t, err, "")
	adapter := StubInterfaceAdapterMock{}
	dbAdapter := StubInterfaceAdapterDBMock{adapter, db}
	manager := ManagerImpl{Adapter: &dbAdapter}
	return dbAdapter, manager
}
