package chaincode

import (
	"github.com/hyperledger/fabric/core/chaincode/shim"
	"github.com/hyperledger/fabric/protos/peer"
	"github.com/stretchr/testify/mock"
	"github.com/hyperledger/fabric/protos/ledger/queryresult"
)

type StateQueryIteratorInterfaceMock struct {
	mock.Mock
}

func (mock *StateQueryIteratorInterfaceMock) Close() error {
	return nil
}

func (mock *StateQueryIteratorInterfaceMock) Next() (*queryresult.KV, error) {
	args := mock.Called()
	return args.Get(0).(*queryresult.KV), args.Error(1)
}

func (mock *StateQueryIteratorInterfaceMock) HasNext() bool {
	args := mock.Called()
	return args.Get(0).(bool)
}

// ChaincodeStubInterfaceAdapterMock chaincode adapter mock interface
type StubInterfaceAdapterMock struct {
	f    string
	args []string
	mock.Mock
}

// GetState
func (mock *StubInterfaceAdapterMock) GetState(key string) ([]byte, error) {
	args := mock.Called(key)
	if args.Get(0) == nil && args.Get(1) == nil {
		return nil, nil
	}
	return args.Get(0).([]byte), args.Error(1)
}

// PutState
func (mock *StubInterfaceAdapterMock) PutState(key string, value []byte) error {
	args := mock.Called(key, value)
	return args.Error(0)
}

// DelState records the specified `key` to be deleted in the writeset of
// the transaction proposal. The `key` and its value will be deleted from
// the ledger when the transaction is validated and successfully committed.
func (mock *StubInterfaceAdapterMock) DelState(key string) error {
	args := mock.Called(key)
	return args.Error(0)
}

// TODO REMOVE >>> DEPRECATED <<<
// GetQueryResult
func (mock *StubInterfaceAdapterMock) GetQueryResult(query string) (shim.StateQueryIteratorInterface, error) {
	args := mock.Called(query)
	return args.Get(0).(shim.StateQueryIteratorInterface), args.Error(1)
}

func (adapter *StubInterfaceAdapterMock) InvokeChaincode(chaincodeName string, args [][]byte, channel string) peer.Response {
	return adapter.Called(chaincodeName, args, channel).Get(0).(peer.Response)
}

func (adapter *StubInterfaceAdapterMock) GetTxID() string {
	return "test_tx_id"
}
