package commissionCC

import (
	"gitlab.smartblocklab.com/smartcontracts/common/model/userCC"
	"github.com/pkg/errors"
	"gitlab.smartblocklab.com/smartcontracts/common/chaincode"
	"gitlab.smartblocklab.com/smartcontracts/common/remote/remote_organization"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/model/requests"
	"encoding/json"
	"reflect"
)

type Commission struct {
	CommissionType    CommissionType      `json:"commission_type,omitempty"`
	CustomerType      userCC.CustomerType `json:"customer_type,omitempty"`
	PartnerCode       string              `json:"partner_code,omitempty"`
	CommissionPercent float64             `json:"commission_percent,omitempty"`
}

func (this Commission) New() chaincode.ValidObject {
	return &Commission{}
}

func (this Commission) CheckBeforeAdd(manager chaincode.IManager) error {
	organization, err := remote_organization.GetOrganizationRemoteManager(manager, invoke.OrganizationChannel, invoke.OrganizationCC).GetOrganization(&requests.Id{Id: this.PartnerCode})
	if err != nil {
		return errors.Errorf("Can't load organization. Error to execute method organizationCC.getOrganization with argument (partnerCode: %s) from organizationchannel",
			this.PartnerCode)
	}

	if organization == nil {
		return errors.Errorf("Organization with partner code %s not found", this.PartnerCode)
	}
	return nil
}

func (this Commission) CheckBeforeUpdate(updatedObject chaincode.ValidObject, manager chaincode.IManager) error {
	organization, err := remote_organization.GetOrganizationRemoteManager(manager, invoke.OrganizationChannel, invoke.OrganizationCC).GetOrganization(&requests.Id{Id: this.PartnerCode})
	if err != nil {
		return errors.Errorf("Can't load organization. Error to execute method organizationCC.getOrganization with argument (partnerCode: %s) from organizationchannel",
			this.PartnerCode)
	}

	if organization == nil {
		return errors.Errorf("Organization with partner code %s not found", this.PartnerCode)
	}
	return nil
}

func (this Commission) Validate() error {
	errorMsg := ""

	if len(this.CommissionType) == 0 {
		errorMsg += "CommissionType is empty; "
	}
	if len(this.CustomerType) == 0 {
		errorMsg += "CustomerType is empty; "
	}

	if !(this.CommissionPercent > 0 && this.CommissionPercent <= 100) {
		errorMsg += "CommissionPercent can be in the range between 0.00 and 100.00; "
	}
	if len(this.PartnerCode) == 0 {
		errorMsg += "PartnerCode is empty; "
	}

	if len(errorMsg) != 0 {
		return errors.New(errorMsg)
	}

	return nil
}

func (this Commission) GetKey(manager chaincode.IManager) []string {
	return []string{this.PartnerCode, this.CustomerType.String(), this.CommissionType.String()}
}

func Unmarshal(data []byte) (*Commission, error) {
	var obj Commission
	err := json.Unmarshal(data, &obj)
	return &obj, err;
}

func Marshal(data *Commission) ([]byte, error) {
	return json.Marshal(data);
}

func Cast(validObject chaincode.ValidObject) (*Commission, error) {
	if validObject == nil {
		return nil, nil
	}

	if reflect.ValueOf(validObject).Kind() != reflect.Ptr {
		return nil, errors.New("check validObjectc is not reference")
	}

	casted, ok := validObject.(*Commission)
	if !ok {
		return nil, errors.New("Problem to get object")
	}

	return casted, nil;
}
