package actions

import (
	log "github.com/sirupsen/logrus"
	"gitlab.smartblocklab.com/smartcontracts/common/invoke"
	"gitlab.smartblocklab.com/smartcontracts/common/logging"
	"strings"
	"errors"
)

type ActionIDsObject struct {
}

type ParameterType int

type Roles string

const (
	UserRole   Roles = "USER"
	SystemRole Roles = "SYSTEM_USER"
	RootRole   Roles = "ROOT_USER"
)

type SecurityPreference struct {
	ChaincodeName string
	FunctionName  string
	ParameterType ParameterType
	Roles         []Roles
}

var (
	logger = log.WithFields(log.Fields{})
)

func (securityPreference SecurityPreference) GetActionID(channelName string) int32 {
	actionID := channelName + securityPreference.ChaincodeName + securityPreference.FunctionName
	logger.Infof("ActionId source: '%s'", actionID)
	var hash int32
	hash = Hash(actionID)
	logger.Infof("ActionId hash: '%d'", hash)
	return hash
}

func Hash(s string) int32 {
	var h int32
	h = 0
	var ln int32
	var li int32
	ln = int32(len(s))
	li = ln - 1 // li means last index
	var i int32
	for i = 0; i < ln; i++ {
		h += int32(s[i]) * Pow(31, li-i)
	}
	return h
}

func Pow(x int32, y int32) int32 {
	p := x
	if y == 0 {
		return 1
	}
	if y == 1 {
		return x
	}
	// if y < 0, panic?
	var i int32
	for i = 1; i < y; i++ {
		p *= x
	}
	return p
}

const UserParameter ParameterType = 1
const ServiceParameter ParameterType = 1 << 2
const ComplexParameter ParameterType = 1 << 3
const UnsecuredParameter ParameterType = 1 << 4

const ping = "ping"
const clearLedger = "clearLedger"

var SecurityPreferenceArray = []SecurityPreference{
	// Common
	// invoke
	SecurityPreference{invoke.AccountCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AuthCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CommissionCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CryptocurrencyCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OrganizationCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserpreferenceCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SystempreferenceCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.DevicepreferenceCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SessionCC, ping, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	SecurityPreference{invoke.AccountCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AuthCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CommissionCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CryptocurrencyCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OrganizationCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserpreferenceCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SystempreferenceCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.DevicepreferenceCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SessionCC, clearLedger, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// AccountCC

	// invoke
	SecurityPreference{invoke.AccountCC, invoke.AddWalletFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.AddAccountFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.ActivateWalletFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.BlockWalletFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.ActivateAccountFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.BlockAccountFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.AccountCC, invoke.GetAccountByCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.VerifyUserAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.CheckAndLoadAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAllUserWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAccountInfoByAccountIDFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetAccountsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AccountCC, invoke.GetWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// MarketaccountCC

	// invoke
	SecurityPreference{invoke.MarketaccountCC, invoke.AddWalletFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.AddAccountFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.ActivateWalletFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.BlockWalletFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.ActivateAccountFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.BlockAccountFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAccountByCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.VerifyUserAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.CheckAndLoadAccountFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAllUserWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAccountInfoByAccountIDFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetAccountsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.MarketaccountCC, invoke.GetWalletsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// AuthCC

	// invoke
	SecurityPreference{invoke.AuthCC, invoke.AddAuthInfoFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.AuthCC, invoke.UpdateAuthInfoFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.AuthCC, invoke.GetAuthInfoFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// CommissionCC

	// invoke
	SecurityPreference{invoke.CommissionCC, invoke.AddCommissionFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CommissionCC, invoke.UpdateCommissionFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.CommissionCC, invoke.GetCommissionFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// CryptocurrencyCC

	// invoke
	SecurityPreference{invoke.CryptocurrencyCC, invoke.AddCryptocurrenciesFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.CryptocurrencyCC, invoke.ClearTransferredCryptocurrencyListFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.CryptocurrencyCC, invoke.GetCryptocurrenciesByDaemonNameFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// OfferCC

	// invoke
	SecurityPreference{invoke.OfferCC, invoke.AddOfferFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.UpdateOfferFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.SetOfferStatusByOperationIdFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.PutOfferPropertiesByOperationIdFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.OfferCC, invoke.GetOfferFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetUserIncomingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetUserOutgoingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetHistoryUserIncomingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetHistoryUserOutgoingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.IsOfferApprovedFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.OfferCC, invoke.GetRequestsByStatusAndCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// ExternalOfferCC

	// invoke
	SecurityPreference{invoke.ExternalOfferCC, invoke.AddOfferFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.UpdateOfferFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.SetOfferStatusByOperationIdFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.PutOfferPropertiesByOperationIdFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetOfferFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetUserIncomingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetUserOutgoingRequestsByStatusFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetHistoryUserIncomingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetHistoryUserOutgoingRequestsFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.IsOfferApprovedFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.ExternalOfferCC, invoke.GetRequestsByStatusAndCurrencyTypeFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// TransactionCC

	// invoke
	SecurityPreference{invoke.TransactionCC, invoke.LockFundsOnAccountFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LockOutFundsOnAccountFunction, ComplexParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.TransferFundsFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.UnlockFundsOnAccountFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LoadCryptocurrencyFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LockTransferToMarketFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.LoadMarketFoundsFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.WithdrawalMarketFundsFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.TransactionCC, invoke.CheckExistsTransactionByListFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.GetUsersTransactionFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.FindLockAmountByOperationIDAndUserIdFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.TransactionCC, invoke.AccountCurrencyBalanceFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// UserCC

	// invoke
	SecurityPreference{invoke.UserCC, invoke.AddRootUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.AddOrganizationRootUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.AddSystemUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.AddServiceUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.AddPublicKeyFunction, ServiceParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.AddUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.RevokeUserPublicKeyFunction, ServiceParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.UserCC, invoke.GetUserFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserCC, invoke.VerifyUserSignatureFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// OrganizationCC

	// invoke
	SecurityPreference{invoke.OrganizationCC, invoke.AddOrganizationFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.OrganizationCC, invoke.GetOrganizationFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// SessionCC

	// invoke
	SecurityPreference{invoke.SessionCC, invoke.AddPreferenceFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SessionCC, invoke.UpdatePreferenceFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.SessionCC, invoke.GetPreferenceFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// UserpreferenceCC

	// invoke
	SecurityPreference{invoke.UserpreferenceCC, invoke.AddPreferenceFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.UserpreferenceCC, invoke.UpdatePreferenceFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.UserpreferenceCC, invoke.GetPreferenceFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// DevicepreferenceCC

	// invoke
	SecurityPreference{invoke.DevicepreferenceCC, invoke.AddPreferenceFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.DevicepreferenceCC, invoke.UpdatePreferenceFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.DevicepreferenceCC, invoke.GetPreferenceFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},

	// SystempreferenceCC

	// invoke
	SecurityPreference{invoke.SystempreferenceCC, invoke.AddPreferenceFunction, UserParameter, []Roles{UserRole}},
	SecurityPreference{invoke.SystempreferenceCC, invoke.UpdatePreferenceFunction, UserParameter, []Roles{UserRole}},
	// query
	SecurityPreference{invoke.SystempreferenceCC, invoke.GetPreferenceFunction, ServiceParameter | UnsecuredParameter, []Roles{UserRole}},
}

func FindSecurityPreferenceByFunctionName(chaincodeName string, functionName string) (SecurityPreference, error) {
	logging.Get().Infof("chaincodeName: %s", chaincodeName)
	logging.Get().Infof("functionName: %s", functionName)
	chaincodeNameLower := firstLetterToLowerCase(chaincodeName)
	functionNameLower := firstLetterToLowerCase(functionName)

	for _, v := range SecurityPreferenceArray {
		if v.ChaincodeName == chaincodeNameLower && v.FunctionName == functionNameLower {
			logging.Get().Infof("found SecurityPreference: ChaincodeName %s, FunctionName %s", v.ChaincodeName, v.FunctionName)
			return v, nil
		}
	}
	return SecurityPreference{}, errors.New("SECURITY_FOR_FUNCTION_NOT_FOUND")
}

func firstLetterToLowerCase(str string) string {
	firstLetter := strings.ToLower(str[0:1])
	replaced := firstLetter + str[1:]

	return replaced
}
